@@ -10,7 +10,6 @@ from asgiref.sync import iscoroutinefunction, sync_to_async
 
 from allauth.account.adapter import get_adapter
 from allauth.account.internal import flows
-from allauth.account.reauthentication import suspend_request
 from allauth.core import context
 from allauth.core.exceptions import (
     ImmediateHttpResponse,
@@ -28,7 +27,9 @@ def AccountMiddleware(get_response):
                 response = await get_response(request)
                 if _should_check_dangling_login(request, response):
                     await _acheck_dangling_login(request)
-                return _accounts_redirect(request, response)
+                if _should_redirect_accounts(request, response):
+                    response = await _aredirect_accounts(request)
+                return response
 
     else:
 
@@ -38,7 +39,9 @@ def AccountMiddleware(get_response):
                 response = get_response(request)
                 if _should_check_dangling_login(request, response):
                     _check_dangling_login(request)
-                return _accounts_redirect(request, response)
+                if _should_redirect_accounts(request, response):
+                    response = _redirect_accounts(request)
+                return response
 
     def process_exception(request, exception):
         if isinstance(exception, ImmediateHttpResponse):
@@ -48,7 +51,7 @@ def AccountMiddleware(get_response):
             methods = get_adapter().get_reauthentication_methods(request.user)
             if methods:
                 redirect_url = methods[0]["url"]
-            return suspend_request(request, redirect_url)
+            return flows.reauthentication.suspend_request(request, redirect_url)
 
     middleware.process_exception = process_exception
     return middleware
@@ -63,7 +66,10 @@ def _should_check_dangling_login(request, response):
         content_type = content_type.partition(";")[0]
     if content_type and content_type != "text/html":
         return False
-    if request.path.startswith(settings.STATIC_URL) or request.path in [
+    # STATIC_URL might be None, as the staticfiles app is not strictly required
+    if (
+        settings.STATIC_URL and request.path.startswith(settings.STATIC_URL)
+    ) or request.path in [
         "/favicon.ico",
         "/robots.txt",
         "/humans.txt",
@@ -84,7 +90,7 @@ async def _acheck_dangling_login(request):
     await sync_to_async(_check_dangling_login)(request)
 
 
-def _accounts_redirect(request, response):
+def _should_redirect_accounts(request, response) -> bool:
     """
     URLs should be hackable. Yet, assuming allauth is included like this...
 
@@ -96,18 +102,30 @@ def _accounts_redirect(request, response):
     is authenticated.
     """
     if response.status_code != 404:
-        return response
+        return False
     try:
         login_path = reverse("account_login")
         email_path = reverse("account_email")
     except NoReverseMatch:
         # Project might have deviated URLs, let's keep out of the way.
-        return response
+        return False
     prefix = os.path.commonprefix([login_path, email_path])
     if len(prefix) <= 1 or prefix != request.path:
-        return response
+        return False
     # If we have a prefix that is not just '/', and that is what our request is
     # pointing to, redirect.
-    return HttpResponseRedirect(
-        email_path if request.user.is_authenticated else login_path
-    )
+    return True
+
+
+async def _aredirect_accounts(request) -> HttpResponseRedirect:
+    email_path = reverse("account_email")
+    login_path = reverse("account_login")
+    user = await request.auser()
+    return HttpResponseRedirect(email_path if user.is_authenticated else login_path)
+
+
+def _redirect_accounts(request) -> HttpResponseRedirect:
+    email_path = reverse("account_email")
+    login_path = reverse("account_login")
+    user = request.user
+    return HttpResponseRedirect(email_path if user.is_authenticated else login_path)
