@@ -4,6 +4,7 @@ from typing import Dict, Optional
 
 from django.conf import settings
 from django.contrib.auth import get_user_model
+from django.contrib.auth.models import AbstractBaseUser
 from django.core import signing
 from django.db import models
 from django.db.models import Q
@@ -128,11 +129,6 @@ class EmailConfirmationMixin:
         if not email_address.verified:
             confirmed = get_adapter().confirm_email(request, email_address)
             if confirmed:
-                signals.email_confirmed.send(
-                    sender=self.__class__,
-                    request=request,
-                    email_address=email_address,
-                )
                 return email_address
 
     def send(self, request=None, signup=False):
@@ -235,6 +231,9 @@ class Login:
     case email verification is optional and we are only logging in).
     """
 
+    # Optional, because we might be prentending logins to prevent user
+    # enumeration.
+    user: Optional[AbstractBaseUser]
     email_verification: app_settings.EmailVerificationMethod
     signal_kwargs: Optional[Dict]
     signup: bool
@@ -276,7 +275,7 @@ class Login:
                 signal_kwargs["sociallogin"] = sociallogin.serialize()
 
         data = {
-            "user_pk": user_pk_to_url_str(self.user),
+            "user_pk": user_pk_to_url_str(self.user) if self.user else None,
             "email_verification": self.email_verification,
             "signup": self.signup,
             "redirect_url": self.redirect_url,
@@ -291,13 +290,12 @@ class Login:
     def deserialize(cls, data):
         from allauth.account.utils import url_str_to_user_pk
 
-        user = (
-            get_user_model()
-            .objects.filter(pk=url_str_to_user_pk(data["user_pk"]))
-            .first()
-        )
-        if user is None:
-            raise ValueError()
+        user = None
+        user_pk = data["user_pk"]
+        if user_pk is not None:
+            user = (
+                get_user_model().objects.filter(pk=url_str_to_user_pk(user_pk)).first()
+            )
         try:
             # :-( Knowledge of the `socialaccount` is entering the `account` app.
             signal_kwargs = data["signal_kwargs"]
@@ -323,7 +321,13 @@ class Login:
 
 
 def get_emailconfirmation_model():
-    if app_settings.EMAIL_CONFIRMATION_HMAC:
+    if app_settings.EMAIL_VERIFICATION_BY_CODE_ENABLED:
+        from allauth.account.internal.flows.email_verification_by_code import (
+            EmailVerificationModel,
+        )
+
+        return EmailVerificationModel
+    elif app_settings.EMAIL_CONFIRMATION_HMAC:
         model = EmailConfirmationHMAC
     else:
         model = EmailConfirmation
