@@ -1,6 +1,6 @@
 from enum import Enum
 
-from allauth.account.stages import EmailVerificationStage
+from allauth.account.stages import EmailVerificationStage, LoginByCodeStage
 
 
 class Client(str, Enum):
@@ -11,7 +11,7 @@ class Client(str, Enum):
 class Flow(str, Enum):
     VERIFY_EMAIL = EmailVerificationStage.key
     LOGIN = "login"
-    LOGIN_BY_CODE = "login_by_code"
+    LOGIN_BY_CODE = LoginByCodeStage.key
     SIGNUP = "signup"
     PROVIDER_REDIRECT = "provider_redirect"
     PROVIDER_SIGNUP = "provider_signup"
