@@ -137,6 +137,12 @@ class AppleTests(OAuth2TestsMixin, TestCase):
             "auth_time": 1234345345,  # not converted automatically by pyjwt
         }
 
+    def test_verify_token(self):
+        id_token = sign_id_token(self.get_apple_id_token_payload())
+        with mocked_response(self.get_mocked_response()):
+            sociallogin = self.provider.verify_token(None, {"id_token": id_token})
+            assert sociallogin.user.email == "test@privaterelay.appleid.com"
+
     def get_login_response_json(self, with_refresh_token=True):
         """
         `with_refresh_token` is not optional for apple, so it's ignored.
@@ -164,6 +170,9 @@ class AppleTests(OAuth2TestsMixin, TestCase):
             200, KEY_SERVER_RESP_JSON, {"content-type": "application/json"}
         )
 
+    def get_expected_to_str(self):
+        return "A B"
+
     def get_complete_parameters(self, auth_request_params):
         """
         Add apple specific response parameters which they include in the
