@@ -5,11 +5,9 @@ from allauth.socialaccount.providers.oauth2.views import (
     OAuth2LoginView,
 )
 
-from .provider import DropboxOAuth2Provider
-
 
 class DropboxOAuth2Adapter(OAuth2Adapter):
-    provider_id = DropboxOAuth2Provider.id
+    provider_id = "dropbox"
     access_token_url = "https://api.dropbox.com/oauth2/token"
     authorize_url = "https://www.dropbox.com/oauth2/authorize"
     profile_url = "https://api.dropbox.com/2/users/get_current_account"
