@@ -5,11 +5,9 @@ from allauth.socialaccount.providers.oauth2.views import (
     OAuth2LoginView,
 )
 
-from .provider import WahooProvider
 
-
-class WahooOauth2Adapter(OAuth2Adapter):
-    provider_id = WahooProvider.id
+class WahooOAuth2Adapter(OAuth2Adapter):
+    provider_id = "wahoo"
     access_token_url = "https://api.wahooligan.com/oauth/token"
     authorize_url = "https://api.wahooligan.com/oauth/authorize"
     profile_url = "https://api.wahooligan.com/v1/user"
@@ -24,5 +22,5 @@ class WahooOauth2Adapter(OAuth2Adapter):
         return self.get_provider().sociallogin_from_response(request, extra_data)
 
 
-oauth2_login = OAuth2LoginView.adapter_view(WahooOauth2Adapter)
-oauth2_callback = OAuth2CallbackView.adapter_view(WahooOauth2Adapter)
+oauth2_login = OAuth2LoginView.adapter_view(WahooOAuth2Adapter)
+oauth2_callback = OAuth2CallbackView.adapter_view(WahooOAuth2Adapter)
